//
// Copyright (c) 2002-2023 WEINZIERL ENGINEERING GmbH
// All rights reserved.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY DAMAGES OR OTHER LIABILITY,
// WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE
//

#ifndef __KDRIVE_EXPRESS_KEYRING_H__
#define __KDRIVE_EXPRESS_KEYRING_H__

#include "kdrive_express_config.h"
#include "kdrive_express_defs.h"

/*!
	\file kdrive_express_keyring.h
	\brief kdriveExpress KNX Keyring API
	\author Copyright (c) 2002-2023 WEINZIERL ENGINEERING GmbH
	\defgroup kdrive_express_keyring kdriveExpress KNX Keyring API
*/

#if KDRIVE_EXPRESS_SECURITY_ENABLED == 1

#ifdef __cplusplus
extern "C"
{
#endif

/*******************************
** Keyring
********************************/

/*!
	Create a new keyring
	\param [in] name the name of the keyring project (as null terminated string)
	\return keyring descriptor if successful
	\ingroup kdrive_express_keyring
*/
kdriveExpress_API int32_t kdrive_keyring_create(const char* name);

/*!
	Imports an ETS keyring project file (*.knxkeys)
	\param [in] filename the ETS keyring filename (as null terminated string)
	\param [in] password the password for encrypt the keyring file (as null terminated string)
	\param [out] keyring the keyring project descriptor
	\ingroup kdrive_express_keyring
*/
kdriveExpress_API error_t kdrive_keyring_import_knxkeys(const char* filename,
        const char* password, int32_t* keyring);

/*!
	Removes a keyring project
	Returns true if successfully released
	false otherwise (i.e. the descriptor wasn't found).
	\param [in] keyring the keyring project descriptor
	\ingroup kdrive_express_keyring
*/
kdriveExpress_API bool_t kdrive_keyring_remove(int32_t keyring);

/*!
	Gets keyring project name
	Null-terminated string in utf8 encoding
	\param [in] keyring the keyring project descriptor
	\param [out] name the project name (as null terminated string)
	\param [in,out] name_length the char array capacity (in) and actual length of the char array (out)
	\ingroup kdrive_express_keyring
*/
kdriveExpress_API error_t kdrive_keyring_get_proj_name(int32_t keyring, char name[], uint32_t* name_length);

/*!
	Gets keyring project creator (e.g. ETS 5.7 (Build 619))
	Null-terminated string in utf8 encoding
	\param [in] keyring the keyring project descriptor
	\param [out] creator the project created by (as null terminated string)
	\param [in,out] creator_length the char array capacity (in) and actual length of the char array (out)
	\ingroup kdrive_express_keyring
*/
kdriveExpress_API error_t kdrive_keyring_get_proj_creator(int32_t keyring,
        char creator[], uint32_t* creator_length);

typedef struct keyring_dev_t
{
	uint16_t ind_addr;                               /*!< KNX individual address */
	uint8_t tool_key[KDRIVE_SEC_KEY_LEN];            /*!< Tool key */
	bool_t has_tool_key;                             /*!< The device has a Tool key */
	char password[KDRIVE_MAX_SEC_PASSWORD_LEN];      /*!< Management password (as null terminated string) */
	bool_t has_mgmt_password;                        /*!< The device has a Management password */
	char auth_password[KDRIVE_MAX_SEC_PASSWORD_LEN]; /*!< Authentication password (as null terminated string) */
	bool_t has_auth_password;                        /*!< The device has a Authentication password */
} keyring_dev_t;

/*!
	Set a device to the keyring.
	If already exist, it will overwrite it.
	\param [in] keyring the keyring project descriptor
	\param [in] device the device structure
	\ingroup kdrive_express_keyring
*/
kdriveExpress_API error_t kdrive_keyring_set_device(int32_t keyring, const keyring_dev_t* device);

/*!
	Get a device from the keyring.
	\param [in] keyring the keyring project descriptor
	\param [in,out] device the device structure, with the individual address of the device to get (in)
	                       and device parameters to fill up (out)
	\ingroup kdrive_express_keyring
*/
kdriveExpress_API error_t kdrive_keyring_get_device(int32_t keyring, keyring_dev_t* device);

typedef struct keyring_tun_t
{
	uint16_t host_ind_addr;                          /*!< KNX individual address of the host */
	uint16_t ind_addr;                               /*!< KNX individual address */
	uint8_t user_id;                                 /*!< The user Id of the connection */
	char password[KDRIVE_MAX_SEC_PASSWORD_LEN];      /*!< The user password (as null terminated string) */
	uint32_t has_user_credentials;                   /*!< The Tunnel has a user and password */
	char auth_password[KDRIVE_MAX_SEC_PASSWORD_LEN]; /*!< Authentication password (as null terminated string) */
	uint32_t has_auth_password;                      /*!< The Tunnel has a authentication password */
} keyring_tun_t;

/*!
	Set a tunneling interface to the keyring.
	If already exist, it will overwrite it.
	\param [in] keyring the keyring project descriptor
	\param [in] tunnel the tunneling interface structure
	\ingroup kdrive_express_keyring
*/
kdriveExpress_API error_t kdrive_keyring_set_tunnel(int32_t keyring, const keyring_tun_t* tunnel);

/*!
	Get a tunneling interface from the keyring.
	\param [in] keyring the keyring project descriptor
	\param [in,out] tunnel the tunneling interface structure, with the individual address of the tunnel to get (in)
	                       and tunnel parameters to fill up (out)
	\ingroup kdrive_express_keyring
*/
kdriveExpress_API error_t kdrive_keyring_get_tunnel(int32_t keyring, keyring_tun_t* tunnel);

/*!
	Removes a device or tunneling interface chosen by it's individual address from the keyring.
	\param [in] keyring the keyring project descriptor
	\param [in] address the individual address of the device or tunneling interface
	\ingroup kdrive_express_keyring
*/
kdriveExpress_API error_t kdrive_keyring_remove_device_tunnel(int32_t keyring, uint16_t address);

/*!
	Set a group address with it's key to the keyring.
	If already exist, it will overwrite it.
	\param [in] keyring the keyring project descriptor
	\param [in] group_addr the group address
	\param [in] key the key for the group address
	\ingroup kdrive_express_keyring
*/
kdriveExpress_API error_t kdrive_keyring_set_group_addr(int32_t keyring,
        uint16_t group_addr,
        const uint8_t key[KDRIVE_SEC_KEY_LEN]);

/*!
	Get the key for a group address from the keyring.
	\param [in] keyring the keyring project descriptor
	\param [in] group_addr the group address
	\param [out] key the key of the group address
	\ingroup kdrive_express_keyring
*/
kdriveExpress_API error_t kdrive_keyring_get_group_addr(int32_t keyring,
        uint16_t group_addr,
        uint8_t key[KDRIVE_SEC_KEY_LEN]);

/*!
	Removes a group address with it's key from the keyring.
	\param [in] keyring the keyring project descriptor
	\param [in] group_addr the group address
	\ingroup kdrive_express_keyring
*/
kdriveExpress_API error_t kdrive_keyring_remove_group_addr(int32_t keyring, uint16_t group_addr);

#ifdef __cplusplus
}
#endif

#endif /* KDRIVE_EXPRESS_SECURITY_ENABLED */

#endif /* __KDRIVE_EXPRESS_KEYRING_H__ */
