//
// Copyright (c) 2002-2023 WEINZIERL ENGINEERING GmbH
// All rights reserved.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY DAMAGES OR OTHER LIABILITY,
// WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE
//

#ifndef __KDRIVE_EXPRESS_MPROP_H__
#define __KDRIVE_EXPRESS_MPROP_H__

#include "kdrive_express_config.h"
#include "kdrive_express_defs.h"

/*!
	\file kdrive_express_mprop.h
	\brief kdriveExpress M_Property API
	\author Copyright (c) 2002-2023 WEINZIERL ENGINEERING GmbH
	\defgroup kdrive_express_mprop kdriveExpress M_Property API
*/

#ifdef __cplusplus
extern "C" {
#endif

/**************************************************************
** Local Service: M_Prop_Read
***************************************************************/

/*!
	Reads a property value from a device via M_Prop_Read service.
	The data buffer must exist (should be pre-allocated by the user) and its size
	should be equal to or greater than the property value to be read.
	The max size of the buffer must be passed in as data_length.
	data_length will be updated to correspond to the length of the value read.
	\note Only available in the Commercial kdriveExpress SDK
	\param [in] ap the access port descriptor
	\param [in] object_type object type
    \param [in] object_instance object instance
	\param [in] prop_id Property ID
	\param [in] nr_of_elems Element Count
	\param [in] start_index Start Index
	\param [out] data the property value read from the device
	\param [in,out] data_length the data array capacity (in) and actual length of the data array (out)
	\return KDRIVE_ERROR_NONE if the service was successful or the error code if an error was raised
	\ingroup kdrive_express_mprop
*/
kdriveExpress_API error_t kdrive_ap_m_prop_read(int32_t ap,
        uint16_t object_type,
        uint16_t object_instance,
        uint16_t prop_id,
        uint8_t nr_of_elems,
        uint16_t start_index,
        uint8_t data[],
        uint32_t* data_length);


/**************************************************************
** Local Service: M_Prop_Write
***************************************************************/

/*!
	Writes the value of a property to a device via M_Prop_Write service.
	This service waits for the response.
	\note Only available in the Commercial kdriveExpress SDK
	\param [in] ap the access port descriptor
	\param [in] object_type object type
    \param [in] object_instance object instance
	\param [in] prop_id property id
	\param [in] nr_of_elems element count
	\param [in] start_index start index
	\param [in] data the data array
	\param [in] data_length the length of the data array
	\param [out] return_code the return code from the device
	\return KDRIVE_ERROR_NONE if the service was successful or the error code if an error was raised
	\ingroup kdrive_express_mprop
*/
kdriveExpress_API error_t kdrive_ap_m_prop_write(int32_t ap,
        uint16_t object_type,
        uint16_t object_instance,
        uint16_t prop_id,
        uint8_t nr_of_elems,
        uint16_t start_index,
        const uint8_t data[],
        uint32_t data_length,
        uint8_t* return_code);

/**************************************************************
** Local Service: M_FuncProp_Command
***************************************************************/

/*!
	Calls a function property of an interface object in a device via M_FuncProp_Command service.
	The output_data buffer must exist (should be pre-allocated by the user) and its size
	should be equal to or greater than the output value of the function response.
	The max size of the buffer must be passed in as output_data_length.
	output_data_length will be updated to correspond to the length of the value response.
	\note Only available in the Commercial kdriveExpress SDK
	\param [in] ap the access port descriptor
	\param [in] object_type object type
    \param [in] object_instance object instance
	\param [in] prop_id property id
	\param [in] input_data the function specific input data
	\param [in] input_data_length the length of the input_data array
	\param [out] return_code return code
	\param [out] output_data the function specific output data
	\param [in,out] output_data_length the output_data array capacity (in) and actual length of the output_data array (out)
	\return KDRIVE_ERROR_NONE if the service was successful or the error code if an error was raised
	\ingroup kdrive_express_mprop
*/
kdriveExpress_API error_t kdrive_ap_m_func_prop_command(int32_t ap,
        uint16_t object_type,
        uint16_t object_instance,
        uint16_t prop_id,
        const uint8_t input_data[],
        uint32_t input_data_length,
        uint8_t* return_code,
        uint8_t output_data[],
        uint32_t* output_data_length);

/**************************************************************
** Device Service: M_FuncProp_StateRead
***************************************************************/

/*!
	Calls a function property of an interface object in a device via M_FuncProp_StateRead service.
	The output_data buffer must exist (should be pre-allocated by the user) and its size
	should be equal to or greater than the output value of the function response.
	The max size of the buffer must be passed in as output_data_length.
	output_data_length will be updated to correspond to the length of the value response.
	\note Only available in the Commercial kdriveExpress SDK
	\param [in] ap the access port descriptor
    \param [in] object_type object type
    \param [in] object_instance object instance
	\param [in] prop_id property id
	\param [in] input_data the function specific input data
	\param [in] input_data_length the length of the input_data array
	\param [out] return_code return code
	\param [out] output_data the function specific output data
	\param [in,out] output_data_length the output_data array capacity (in) and actual length of the output_data array (out)
	\return KDRIVE_ERROR_NONE if the service was successful or the error code if an error was raised
	\ingroup kdrive_express_mprop
*/
kdriveExpress_API error_t kdrive_ap_m_func_prop_state_read(int32_t ap,
        uint16_t object_type,
        uint16_t object_instance,
        uint16_t prop_id,
        const uint8_t input_data[],
        uint32_t input_data_length,
        uint8_t* return_code,
        uint8_t output_data[],
        uint32_t* output_data_length);

#ifdef __cplusplus
}
#endif

#endif /* __KDRIVE_EXPRESS_MPROP_H__ */
